document.addEventListener('DOMContentLoaded', () => {
    const textInput = document.getElementById('textInput');
    const fontSizeInput = document.getElementById('fontSize');

    const addTextBtn = document.getElementById('addTextBtn');
    const imageUpload = document.getElementById('imageUpload');
    const addImageBtn = document.getElementById('addImageBtn');
    const labelWidthInput = document.getElementById('labelWidth');
    const labelHeightInput = document.getElementById('labelHeight');
    const applyLabelSizeBtn = document.getElementById('applyLabelSizeBtn');
    const printLabelBtn = document.getElementById('printLabelBtn');
    const clearCanvasBtn = document.getElementById('clearCanvasBtn');

    // 表格输入元素
    const tableRowsInput = document.getElementById('tableRows');
    const tableColsInput = document.getElementById('tableCols');
    const addTableBtn = document.getElementById('addTableBtn');

    // 打印份数输入元素
    const printCopiesInput = document.getElementById('printCopies');

    const messageArea = document.getElementById('message-area');
    const konvaContainer = document.getElementById('konva-container');
    const contentElementsList = document.getElementById('content-elements-list'); 

    // DPI 常量与后端保持一致
    const DPI_PRINTER_FRONT = 203; 
    const MM_TO_PIXEL_RATIO_ACCURATE = DPI_PRINTER_FRONT / 25.4; 

    let currentLabelWidthMM = parseFloat(localStorage.getItem('labelWidthMM')) || 40;
    let currentLabelHeightMM = parseFloat(localStorage.getItem('labelHeightMM')) || 30;

    let currentCanvasWidthPX = Math.round(currentLabelWidthMM * MM_TO_PIXEL_RATIO_ACCURATE);
    let currentCanvasHeightPX = Math.round(currentLabelHeightMM * MM_TO_PIXEL_RATIO_ACCURATE);

    const stage = new Konva.Stage({
        container: 'konva-container',
        width: currentCanvasWidthPX,
        height: currentCanvasHeightPX,
    });

    const layer = new Konva.Layer();
    stage.add(layer);

    let selectedNode = null;
    let tr = new Konva.Transformer();
    layer.add(tr);

    const elementsOnCanvas = []; 

    function updateElementsList() {
        contentElementsList.innerHTML = ''; 
        if (elementsOnCanvas.length === 0) {
            contentElementsList.innerHTML = '<p class="list-placeholder">在此显示已添加的文字和图片列表</p>';
        } else {
            const ul = document.createElement('ul');
            elementsOnCanvas.forEach(el => {
                const li = document.createElement('li');
                li.className = 'content-list-item rounded-md shadow-sm'; 
                li.id = `list-item-${el.id}`; 

                let contentText = '';
                if (el.type === 'text') {
                    contentText = el.node.text();
                    li.innerHTML = `<span class="item-content">文字: "${contentText}"</span>`;
                } else if (el.type === 'image') {
                    contentText = el.imageName || '图片';
                    li.innerHTML = `<span class="item-content">图片: ${contentText}</span>`;
                } else if (el.type === 'table') { 
                    const scaledWidth = (el.node.width() * el.node.scaleX()).toFixed(0);
                    const scaledHeight = (el.node.height() * el.node.scaleY()).toFixed(0);
                    // 在列表显示时，优先显示原始行/列数，更直观
                    const originalRows = el.node.getAttr('rows');
                    const originalCols = el.node.getAttr('cols');
                    contentText = `表格: ${originalRows}行 x ${originalCols}列 (${scaledWidth}x${scaledHeight}px)`;
                    li.innerHTML = `<span class="item-content">${contentText}</span>`;
                }

                const deleteBtn = document.createElement('button');
                deleteBtn.textContent = '删除';
                deleteBtn.className = 'delete-btn'; 
                deleteBtn.onclick = () => deleteElement(el.id);
                li.appendChild(deleteBtn);
                ul.appendChild(li);
                el.listItem = li; 
            });
            contentElementsList.appendChild(ul);
        }
    }

    function deleteElement(elementId) {
        const index = elementsOnCanvas.findIndex(el => el.id === elementId);
        if (index > -1) {
            const elementToRemove = elementsOnCanvas[index];
            elementToRemove.node.destroy(); 
            if (elementToRemove.listItem) {
                elementToRemove.listItem.remove(); 
            }
            elementsOnCanvas.splice(index, 1); 
            layer.draw(); 
            tr.nodes([]); 
            updateElementsList(); 
            showMessage('元素已删除。', 'info');
        }
    }

    stage.on('click tap', (e) => {
        if (e.target === stage || e.target === layer) {
            tr.nodes([]); 
            selectedNode = null;
        } else {
            selectedNode = e.target;
            let targetNode = e.target;
            while(targetNode && !targetNode.hasName('labelElement') && targetNode !== layer) {
                targetNode = targetNode.parent;
            }
            if (targetNode && targetNode.hasName('labelElement')) {
                tr.nodes([targetNode]); 
                selectedNode = targetNode; 
            } else {
                tr.nodes([]); 
                selectedNode = null;
            }
        }
        layer.batchDraw();
    });

    layer.on('dblclick dbltap', (e) => {
        if (e.target.getClassName() === 'Text') {
            const textNode = e.target;
            textNode.hide(); 
            tr.hide(); 

            const textPosition = textNode.absolutePosition();
            const stageBox = konvaContainer.getBoundingClientRect(); 

            const textarea = document.createElement('textarea');
            document.body.appendChild(textarea);

            textarea.value = textNode.text();
            textarea.style.position = 'absolute';
            textarea.style.top = stageBox.top + window.scrollY + textPosition.y + 'px'; 
            textarea.style.left = stageBox.left + window.scrollX + textPosition.x + 'px'; 
            
            const textRect = textNode.getClientRect(); 
            textarea.style.width = textRect.width + 'px';
            textarea.style.height = textRect.height + 'px';
            
            textarea.style.fontSize = textNode.fontSize() * textNode.scaleX() + 'px';
            textarea.style.border = '1px solid #ccc';
            textarea.style.padding = '5px';
            textarea.style.margin = '0';
            textarea.style.overflow = 'hidden';
            textarea.style.background = 'white';
            textarea.style.resize = 'none';
            textarea.style.zIndex = '9999';
            textarea.style.lineHeight = textNode.lineHeight();
            textarea.style.fontFamily = textNode.fontFamily();
            textarea.style.textAlign = textNode.align();
            textarea.focus();

            let blurTimeout; 

            function removeTextarea() {
                clearTimeout(blurTimeout); 
                textarea.remove();
                textNode.show();
                tr.show();
                tr.forceUpdate(); 
                layer.batchDraw();
            }

            textarea.addEventListener('keydown', (e) => {
                if (e.key === 'Enter' && !e.shiftKey) {
                    e.preventDefault(); 
                    textNode.text(textarea.value);
                    removeTextarea(); 
                    updateElementsList(); 
                }
                if (e.key === 'Escape') {
                    removeTextarea(); 
                }
            });

            textarea.addEventListener('blur', () => {
                blurTimeout = setTimeout(() => {
                    textNode.text(textarea.value);
                    removeTextarea();
                    updateElementsList(); 
                }, 100); 
            });
        }
    });

    applyLabelSizeBtn.addEventListener('click', () => {
        const newWidthMM = parseFloat(labelWidthInput.value);
        const newHeightMM = parseFloat(labelHeightInput.value);

        if (isNaN(newWidthMM) || isNaN(newHeightMM) || newWidthMM <= 0 || newHeightMM <= 0) {
            showMessage('请输入有效的标签宽度和高度。', 'error');
            return;
        }

        currentLabelWidthMM = newWidthMM;
        currentLabelHeightMM = newHeightMM;

        localStorage.setItem('labelWidthMM', newWidthMM);
        localStorage.setItem('labelHeightMM', newHeightMM);

        currentCanvasWidthPX = Math.round(currentLabelWidthMM * MM_TO_PIXEL_RATIO_ACCURATE);
        currentCanvasHeightPX = Math.round(currentLabelHeightMM * MM_TO_PIXEL_RATIO_ACCURATE);

        stage.width(currentCanvasWidthPX);
        stage.height(currentCanvasHeightPX);
        layer.draw(); 
        showMessage(`标签尺寸已更新为 ${newWidthMM}mm x ${newHeightMM}mm。`, 'success');
    });

    addTextBtn.addEventListener('click', () => {
        const textContent = textInput.value.trim();
        const fontSize = parseInt(fontSizeInput.value);

        if (!textContent) {
            showMessage('请输入要添加的文字。', 'error');
            return;
        }
        if (isNaN(fontSize) || fontSize <= 0) {
            showMessage('请输入有效的字体大小。', 'error');
            return;
        }

        const textNodeId = `element-${Date.now()}`; 
        const textNode = new Konva.Text({
            x: 10,
            y: 10,
            text: textContent,
            fontSize: fontSize,
            fontFamily: 'Microsoft YaHei, Arial', 
            fill: 'black', 
            draggable: true,
            padding: 5,
            align: 'left',
            verticalAlign: 'top',
            id: textNodeId, 
            name: 'labelElement' 
        });

        layer.add(textNode);
        elementsOnCanvas.push({ id: textNodeId, type: 'text', node: textNode });
        layer.draw();
        tr.nodes([textNode]); 
        updateElementsList(); 
        showMessage('文字已添加到画布。', 'success');
    });

    addImageBtn.addEventListener('click', () => {
        const file = imageUpload.files[0];
        if (!file) {
            showMessage('请选择一张图片。', 'error');
            return;
        }

        const reader = new FileReader();
        reader.onload = function (e) {
            const img = new Image();
            img.onload = function () {
                const imageNodeId = `element-${Date.now()}`; 
                const konvaImage = new Konva.Image({
                    x: 10,
                    y: 10,
                    image: img,
                    width: img.width,
                    height: img.height,
                    draggable: true,
                    id: imageNodeId, 
                    name: 'labelElement' 
                });

                const canvasWidth = stage.width();
                const canvasHeight = stage.height();

                let initialImageWidth = konvaImage.width();
                let initialImageHeight = konvaImage.height();

                if (initialImageWidth > canvasWidth || initialImageHeight > canvasHeight) {
                    const scaleX = canvasWidth / initialImageWidth;
                    const scaleY = canvasHeight / initialImageHeight;
                    const scaleToFit = Math.min(scaleX, scaleY); 

                    konvaImage.width(initialImageWidth * scaleToFit);
                    konvaImage.height(initialImageHeight * scaleToFit);
                    
                    konvaImage.x((canvasWidth - konvaImage.width()) / 2);
                    konvaImage.y((canvasHeight - konvaImage.height()) / 2);

                } else {
                    konvaImage.x(10);
                    konvaImage.y(10);
                }

                layer.add(konvaImage);
                elementsOnCanvas.push({ id: imageNodeId, type: 'image', node: konvaImage, originalSrc: e.target.result, imageName: file.name });
                layer.draw();
                tr.nodes([konvaImage]); 
                updateElementsList(); 
                showMessage('图片已添加到画布。', 'success');
            };
            img.src = e.target.result;
        };
        reader.readAsDataURL(file);
    });

    // 添加表格功能
    addTableBtn.addEventListener('click', () => {
        const rows = parseInt(tableRowsInput.value);
        const cols = parseInt(tableColsInput.value);

        if (isNaN(rows) || rows <= 0 || isNaN(cols) || cols <= 0) {
            showMessage('请输入有效的表格行数和列数。', 'error');
            return;
        }

        const stageWidth = stage.width();
        const stageHeight = stage.height();
        
        const tableWidth = Math.max(50, Math.min(stageWidth * 0.8, 200)); 
        const tableHeight = Math.max(50, Math.min(stageHeight * 0.8, 100)); 

        console.log("Adding table. Rows:", rows, "Cols:", cols);
        console.log("Stage dimensions:", stageWidth, stageHeight);
        console.log("Calculated table dimensions:", tableWidth, tableHeight);

        const tableNodeId = `element-${Date.now()}`;
        const tableGroup = new Konva.Group({
            x: (stageWidth - tableWidth) / 2, 
            y: (stageHeight - tableHeight) / 2, 
            width: tableWidth,
            height: tableHeight,
            draggable: true,
            id: tableNodeId,
            name: 'labelElement', 
            rows: rows, // 确保原始行数被正确存储
            cols: cols, // 确保原始列数被正确存储
        });

        // 添加一个透明背景矩形作为组的命中区域和视觉框
        tableGroup.add(new Konva.Rect({
            x: 0, y: 0,
            width: tableWidth,
            height: tableHeight,
            fill: 'rgba(0,0,0,0.01)', // 几乎透明，但提供可点击区域
            stroke: 'black', 
            strokeWidth: 1, 
        }));

        // 绘制行分隔线
        for (let i = 1; i < rows; i++) {
            tableGroup.add(new Konva.Line({
                points: [0, i * (tableHeight / rows), tableWidth, i * (tableHeight / rows)],
                stroke: 'black',
                strokeWidth: 1,
            }));
        }

        // 绘制列分隔线
        for (let i = 1; i < cols; i++) {
            tableGroup.add(new Konva.Line({
                points: [i * (tableWidth / cols), 0, i * (tableWidth / cols), tableHeight],
                stroke: 'black',
                strokeWidth: 1,
            }));
        }

        layer.add(tableGroup); 
        elementsOnCanvas.push({ id: tableNodeId, type: 'table', node: tableGroup });
        layer.draw(); 
        tr.nodes([tableGroup]); 
        updateElementsList(); 
        showMessage(`表格 (${rows}行x${cols}列) 已添加到画布。`, 'success');
        console.log("Table group added:", tableGroup);
        console.log("Table group children:", tableGroup.children);
    });

    clearCanvasBtn.addEventListener('click', () => {
        layer.removeChildren(); 
        elementsOnCanvas.length = 0; 
        tr.nodes([]); 
        layer.add(tr); 
        layer.draw();
        updateElementsList(); 
        showMessage('画布已清空。', 'info');
    });

    printLabelBtn.addEventListener('click', async () => {
        showMessage('准备发送打印请求...', 'info');

        const elementsToPrint = [];
        elementsOnCanvas.forEach(el => {
            const node = el.node; // Konva Node object
            const elementData = {
                id: node.id(),
                type: el.type,
                x: node.x(),
                y: node.y(),
                width: node.width(), 
                height: node.height(), 
                scaleX: node.scaleX(),
                scaleY: node.scaleY(),
                rotation: node.rotation(),
            };

            if (el.type === 'text') {
                elementData.content = node.text();
                // 确保 fontSize 反映了用户设置的初始值，而不是 Konva 内部缩放后的值
                elementData.fontSize = node.fontSize(); 
                // 为了确保后端能够正确地对文字进行裁剪以适应前端KonvaText的尺寸，
                // 我们还需要将KonvaText的实际渲染宽度和高度传递过去
                const textBoundingBox = node.getClientRect();
                elementData.renderedWidth = textBoundingBox.width;
                elementData.renderedHeight = textBoundingBox.height;

            } else if (el.type === 'image') {
                elementData.src = el.originalSrc; 
            } else if (el.type === 'table') { 
                // 关键修复：使用 getAttr() 确保获取到原始的 rows 和 cols 属性
                elementData.rows = node.getAttr('rows');
                elementData.cols = node.getAttr('cols');
                // 对于表格，width和height直接使用group的当前渲染尺寸
                elementData.width = node.width(); 
                elementData.height = node.height();
            }
            elementsToPrint.push(elementData);
        });

        // 获取打印份数
        const copies = parseInt(printCopiesInput.value);
        if (isNaN(copies) || copies <= 0) {
            showMessage('请输入有效的打印份数（至少为1）。', 'error');
            return;
        }


        console.log("前端收集到的元素数据:", JSON.stringify(elementsToPrint, null, 2));
        if (elementsToPrint.length === 0) {
            showMessage('没有可打印的内容。请先添加文字或图片。', 'error');
            return;
        }

        const printData = {
            label_width_mm: currentLabelWidthMM,
            label_height_mm: currentLabelHeightMM,
            canvas_width_front: currentCanvasWidthPX, 
            canvas_height_front: currentCanvasHeightPX, 
            elements: elementsToPrint,
            copies: copies // 添加打印份数
        };

        console.log("准备发送数据到后端进行WYSIWYG打印:", printData);
        showMessage('打印请求发送中...', 'info');

        try {
            const response = await fetch('/print_label_wysiwyg', {
                method: 'POST',
                headers: {
                    'Content-Type': 'application/json',
                },
                body: JSON.stringify(printData),
            });

            const result = await response.json();
            if (response.ok) {
                showMessage('打印请求成功：' + result.message, 'success');
                console.log('Success:', result);
            } else {
                showMessage('打印请求失败：' + result.message, 'error');
                console.error('Error:', result);
            }
        } catch (error) {
            console.error('网络请求或服务器错误:', error);
            showMessage('网络请求或服务器错误。', 'error');
        }
    });

    let messageTimeout;
    function showMessage(message, type) {
        if (messageArea) {
            clearTimeout(messageTimeout); 
            messageArea.textContent = message;
            messageArea.className = `message-area ${type} show`; 

            messageTimeout = setTimeout(() => {
                messageArea.classList.remove('show');
            }, 3000);
        } else {
            console.warn("Message area element not found. Message:", message, type);
        }
    }

    labelWidthInput.value = currentLabelWidthMM;
    labelHeightInput.value = currentLabelHeightMM;
    applyLabelSizeBtn.click(); 

    // 设置打印份数默认值
    printCopiesInput.value = 1;

    updateElementsList(); 

    const headerLogo = document.querySelector('header img');
    if (headerLogo) {
        headerLogo.src = "{{ url_for('static', filename='logo.png') }}";
        headerLogo.onerror = function() {
            console.warn("Logo image not found or failed to load. Using placeholder.");
            headerLogo.alt = "公司Logo (未找到)";
            headerLogo.style.display = 'none';
        };
    }

});
